﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.CompilerServices;


namespace HSVSpace
{
 
    internal struct RGBColorAVR
    {
        public Byte R;
        public Byte G;
        public Byte B;

        public void RGBColor(Byte R, Byte G, Byte B)
        {
            this.R = R;
            this.G = G;
            this.B = B;
        }

        public byte ByteR
        {
            get { return R; }
            set { R = value; }
        }

        public byte ByteG
        {
            get { return G; }
            set { G = value; }
        }
        
        public byte ByteB
        {
            get { return B; }
            set { B = value; }
        }
    }


    internal struct HSVColorAVR
    {
        public Byte H;
        public Byte S;
        public Byte V;

        public void HSVColor(Byte H, Byte S, Byte V)
        {
            this.H = H;
            this.S = S;
            this.V = V;
        }
    }


    internal static class HSV2RGBAVR
    {
        [MethodImpl(MethodImplOptions.Synchronized)]
        public static RGBColorAVR Convert(HSVColorAVR HSV)
        {
            Byte H = HSV.H;
            Byte S = HSV.S;
            Byte V = HSV.V;
            
            RGBColorAVR RGB = new RGBColorAVR();

            if (HSV.S == 0)
            {
                RGB.R = RGB.G = RGB.B = V;
            }
            else
            {
                Byte i, f, p, q, t;

                UInt16 tmpH = (UInt16)(H * 6);
                i = (Byte)(tmpH >> 8);
                f = unchecked((Byte)tmpH);


                // p = (V * (255 - S)) >> 8 (needed in each region)
                p = (Byte)(255 - S);
                p = (Byte)((V * p) >> 8);

                // q = (V * (255 - ((S * f) >> 8))) >> 8
                q = (Byte)((S * f) >> 8);
                q = (Byte)(255 - q);
                q = (Byte)((V * q) >> 8);

                // t = (V * (255 - ((S * (255 - f)) >> 8))) >> 8
                t = (Byte)(255 - f);
                t = (Byte)((S * t) >> 8);
                t = (Byte)(255 - t);
                t = (Byte)((V * t) >> 8);


                if (i == 0)
                {
                    // r = V; g = t; b = p;
                    RGB.R = V;
                    RGB.G = t;
                    RGB.B = p;
                }
                else if (i == 1)
                {
                    // r = q; g = V; b = p;
                    RGB.R = q;
                    RGB.G = V;
                    RGB.B = p;
                }
                else if (i == 2)
                {
                    // r = p; g = V; b = t;
                    RGB.R = p;
                    RGB.G = V;
                    RGB.B = t;
                }
                else if (i == 3)
                {
                    // r = p; g = q; b = V;
                    RGB.R = p;
                    RGB.G = q;
                    RGB.B = V;
                }
                else if (i == 4)
                {
                    //r = t; g = p; b = V;
                    RGB.R = t;
                    RGB.G = p;
                    RGB.B = V;
                }
                else
                {
                    // r = V; g = p; b = q;
                    RGB.R = V;
                    RGB.G = p;
                    RGB.B = q;
                }
            }

            return RGB;
        }

    
    }
}
